"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const validateNuxtConfig_1 = __importDefault(require("./utils/validateNuxtConfig"));
const updateDependencies_1 = require("./utils/updateDependencies");
const deploy_1 = require("@edgio/core/deploy");
const FrameworkBuildError_1 = __importDefault(require("@edgio/core/errors/FrameworkBuildError"));
const NuxtRoutes_1 = require("./router/NuxtRoutes");
const nft_1 = require("@vercel/nft");
const fs_1 = __importDefault(require("fs"));
const { loadNuxtConfig } = require('@nuxt/config');
const appDir = process.cwd();
const builder = new deploy_1.DeploymentBuilder(appDir);
function build(options) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        builder.clearPreviousBuildOutput();
        const { skipFramework } = options;
        const config = yield loadNuxtConfig();
        const nuxtDirName = (_a = config.buildDir) !== null && _a !== void 0 ? _a : '.nuxt';
        const nuxtDir = (0, path_1.join)(appDir, nuxtDirName);
        // Nuxt will produce static assets that need to be served by NuxtRoutes if target='static' or the config has a generate block,
        // for example, to generate a 404.html
        const isStatic = config.target === 'static';
        const lambdaAssetCopyOptions = {};
        if (!skipFramework) {
            // clear .nuxt directory
            builder.emptyDirSync(nuxtDir);
            // ensure that nuxt.config.js exists and has mode: universal
            yield (0, validateNuxtConfig_1.default)(appDir);
            // ensure the dependencies are defined
            yield (0, updateDependencies_1.validateDependencies)();
            // if static, we dont need to run the build / create a server - the CDN / edgio will serve the files
            const command = isStatic ? 'npx nuxt generate' : 'npx nuxt build --standalone';
            try {
                yield builder.exec(command);
            }
            catch (e) {
                throw new FrameworkBuildError_1.default('Nuxt.js', command, e);
            }
        }
        // edgio-nuxt.config.json
        const edgioConfigPath = (0, path_1.join)(appDir, NuxtRoutes_1.EDGIO_NUXT_CONFIG_PATH);
        builder.writeFileSync(edgioConfigPath, JSON.stringify(config));
        builder.addJSAsset(edgioConfigPath);
        if (isStatic) {
            // static pages
            builder.addStaticAsset((0, path_1.join)(appDir, 'dist'));
        }
        else {
            builder
                // nuxt.js client assets
                .addStaticAsset((0, path_1.join)(appDir, nuxtDirName, 'dist', 'client'), undefined, NuxtRoutes_1.browserAssetOpts)
                // Vue components
                .addJSAsset((0, path_1.join)(appDir, nuxtDirName, 'dist', 'server'), undefined, lambdaAssetCopyOptions)
                // Vue components
                .addJSAsset((0, path_1.join)(appDir, nuxtDirName, 'routes.json'));
            // middleware needs to be copied, otherwise we wont have access to it on prod build
            config.serverMiddleware &&
                Object.values(config.serverMiddleware).forEach(value => 
                // we strip tilde, as it means 'root of the application' in this context anyways, and we cant copy otherwise
                builder.addJSAsset((0, path_1.join)(appDir, value).replace('~/', '')));
            const tsConfigPath = (0, path_1.join)(appDir, 'nuxt.config.ts');
            // checking to avoid warnings in console
            fs_1.default.existsSync(tsConfigPath)
                ? builder.addJSAsset(tsConfigPath)
                : builder.addJSAsset((0, path_1.join)(appDir, 'nuxt.config.js'));
        }
        yield builder.build();
        if (!isStatic) {
            // Add deps for lambda function to be able to programatically invoke server start
            // not needed for static, as the server isnt needed for it
            const files = ['./node_modules/@nuxt/core/dist/core.js'];
            const { fileList } = yield (0, nft_1.nodeFileTrace)(files);
            Array.from(fileList)
                .filter(file => file.indexOf('node_modules') === 0)
                .forEach(file => builder.copySync(file, (0, path_1.join)(builder.buildDir, 'lambda', file)));
        }
        if (getEdgioSourceMapsValue(config) === false) {
            console.log(`> Found edgioSourceMaps set to false`);
            console.log(`> Deleting .map files from lambda folder`);
            builder.deleteMapFiles(builder.jsAppDir);
        }
    });
}
exports.default = build;
/**
 * Returns edgioSourceMaps value from Nuxt config
 * @param config NuxtConfig
 * @return The value of edgioSourceMaps option or null when it's missing
 */
function getEdgioSourceMapsValue(config) {
    var _a;
    let value = null;
    ((_a = config === null || config === void 0 ? void 0 : config.buildModules) !== null && _a !== void 0 ? _a : []).forEach((module) => {
        if (Array.isArray(module)) {
            const moduleOption = module.find(item => typeof item === 'object' && Object.keys(item).includes('edgioSourceMaps'));
            if (moduleOption) {
                value = moduleOption.edgioSourceMaps;
            }
        }
    });
    return value;
}
